/*
*
*  Created on: 17/11/2009
*      Author: Jian Yin Shen
*/
#ifndef _RTCMA_MWPROGRESSIVE_TASK_H_
#define _RTCMA_MWPROGRESSIVE_TASK_H_

#include "MWThread.h"
//#include "MWStateMachine.h"

#include <vector>
#include <deque>
#include <stdexcept>

namespace RtcmaCommMiddleware 
{
	namespace thread {
		class MWProgressiveTask;


		//
		//Defines a state machine for progressive task.
		//
		enum EventTag {
			EventNoEvent,
			EventStartTag,
			EventExceptionThrownTag,
			EventRunFinishedTag,
			EventRunContinueTag,
			EventInitFinishedTag,
			EventInitContinueTag,
			EventPauseTag,
			EventResumeTag,
			EventStopTag, 
			EventFinalizationFinishedTag,
			EventFinalizationContinueTag
		};

		struct event_start {
		};

		struct event_exception_thrown {
		public:
			std::string message;
			event_exception_thrown(const std::string& msg) : message(msg) {}
		};

		struct event_run_finished {
		};

		struct event_run_continue {
		};

		struct event_init_finished {
		};

		struct event_init_continue {
		};

		struct event_pause {
		};

		struct event_resume {
		};

		struct event_stop {
		};

		struct event_finalization_finished {
		};

		struct event_finalization_continue {
		};

		/**
		 * \brief Doxygen stuck on this class
		 *
		 */
		class MWProgressiveTask : public MWTask, public state_machine<MWProgressiveTask> {

		public:
			enum states {
				Ready,
				Initializing,
				Running,
				Finalizing,
				Paused,
				Stopped,
				StoppedInconsistant,
				initial_state = Ready
			};
		private:

			ObserverVec::iterator findObserver(MWProgressiveTaskObserver* obs);

			//State transition actions
			void doThrowException(const event_exception_thrown& e);
			void doThrowException(const event_stop& e);

			void doInitializeStep(const event_start& e);
			void doInitializeStep(const event_init_continue& e);

			void doExecuteStep(const event_init_finished& e);
			void doExecuteStep(const event_run_continue& e);

			void doFinalizeStep(const event_stop& e);
			void doFinalizeStep(const event_run_finished& e);
			void doFinalizeStep(const event_finalization_continue& e);

			void doPause(const event_pause& e);
			void doResume(const event_resume& e);
			void doStop(const event_finalization_finished& e);

			typedef MWProgressiveTask p;
			//Transition table
			
			/**
			 * The definition of this struct stucks doxygen.
			 */
			struct transition_table : mpl::vector17<
			
				TransitionList <Ready, event_start, Initializing, &p::doInitializeStep>,

				TransitionList <Initializing, event_init_finished, Running, &p::doExecuteStep>,
				TransitionList <Initializing, event_exception_thrown, StoppedInconsistant, &p::doThrowException>,
				TransitionList <Initializing, event_stop, Finalizing, &p::doFinalizeStep>,
				TransitionList <Initializing, event_init_continue, Initializing, &p::doInitializeStep>, //to self

				TransitionList <Running, event_run_finished, Finalizing, &p::doFinalizeStep>,
				TransitionList <Running, event_exception_thrown, StoppedInconsistant, &p::doThrowException>,
				TransitionList <Running, event_stop, Finalizing, &p::doFinalizeStep>,
				TransitionList <Running, event_pause, Paused, &p::doPause>,
				TransitionList <Running, event_run_continue, Running, &p::doExecuteStep>, //to self

				TransitionList <Paused, event_exception_thrown, StoppedInconsistant, &p::doThrowException>,
				TransitionList <Paused, event_resume, Running, &p::doResume>,
				TransitionList <Paused, event_stop, Finalizing, &p::doFinalizeStep>,
	
				TransitionList <Finalizing, event_exception_thrown, StoppedInconsistant, &p::doThrowException>,
				TransitionList <Finalizing, event_stop, StoppedInconsistant, &p::doThrowException>,
				TransitionList <Finalizing, event_finalization_finished, Stopped, &p::doStop>,
				TransitionList <Finalizing, event_finalization_continue, Finalizing, &p::doFinalizeStep> //to self
			> {};



		}; //MWProgressiveTask
	}
}

#endif
